/*
 * GEMClip, a Clipboard Manager for FreeMiNT and MagiC.
 *
 * Copyright (C) 2011 Jean-Franois Lemaire (jflemaire@skynet.be)
 *
 * This file is part of GEMClip.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <gemx.h>
#include <mint/ostruct.h>

/* ResourceMaster-Generated Header File */
#include "gemclipe.rsh"

#define MAX(a, b) ((a) > (b) ? (a) : (b))
#define MIN(a, b) ((a) < (b) ? (a) : (b))

/*  Keycodes. */
enum { SC_CTRL_C = 0x2e03,
	   SC_CTRL_V = 0x2f16,
	   SC_CTRL_X = 0x2d18 };

enum { SC_SPACE = 0x3920,
	   SC_ESC = 0x011b,
	   SC_BACKSPACE = 0x0e08,
	   SC_DEL = 0x537f,
	   SC_TAB = 0x0f09,
	   SC_NMPAD_ENTER = 0x720d,
	   SC_HELP = 0x6200,
	   SC_UNDO = 0x6100,
	   SC_INSERT = 0x5200,
	   SC_RETURN = 0x1c0d,
	   SC_CLRHOME = 0x4700,
	   SC_SHFT_CLRHOME = 0x4737,
	   SC_CTRL_CLRHOME = 0x7700,
	   SC_UPARROW = 0x4800,
	   SC_DNARROW = 0x5000,
	   SC_LFARROW = 0x4b00,
	   SC_RTARROW = 0x4d00,
	   SC_SHFT_UPARROW = 0x4838,
	   SC_SHFT_DNARROW = 0x5032,
	   SC_SHFT_LFARROW = 0x4b34,
	   SC_SHFT_RTARROW = 0x4d36 };

/* 
 * Left/Right arrows have different scan with ctrl,
 * while up/down arrows are unaffected by ctrl
 */
enum { SC_CTRL_LFARROW = 0x7300,
	   SC_CTRL_RTARROW = 0x7400,
	   SC_PGUP = 0x4900,
	   SC_PGDN = 0x5100,
	   SC_END = 0x4f00 };

/* GEMDOS Error Codes. */
enum { E_OK = 0, EPTHNF = -34, ENHNDL = -35, EACCDN = -36, ENMFIL = -49 };
enum { SCRAP = 0, NOSCRAP = -1, NCLIPS = 100, WARNCLIPS = 12 }; // 1st item in the clips array is scrap.txt.
enum { FALSE, TRUE };
enum { MINT, MAGIC };
enum { WINAPP, WINABT, WINCFG, WININF };

/* General Buffer Sizes */
enum { SBUFFER = 32, MBUFFER = 128, LBUFFER = 256 };

/* List Box Display Mode Settings */
enum { LIST_TEXT, LIST_INFO, LIST_NAME, LIST_DATE, LIST_SIZE, LIST_TOP, LIST_BOTTOM};

typedef struct gemclip GEMCLIP;
typedef struct clips CLIPS;
typedef struct userconf USERCONF;
typedef struct window WINDOW;
typedef struct text TEXT;
typedef struct bubbles  BUBBLES;

struct gemclip {
	OBJECT *tree;		// Manager main window.
	OBJECT *menu;		// Manager window menu.
	WINDOW *window;
	CLIPS  *clips[NCLIPS]; // 99 user clips + scrap.txt.
	int     topclip;	// The # of the clip currently on top of the listbox.
	char    path[LBUFFER]; // Application path.
	int     suspend;	// Suspend the Manager. TRUE or FALSE.
	int     nclips;		// # of clips in the clipboard.
	int     clipn;		// Index of selected clip (may be scrap.txt or 0) in the clips array.
	int     selected;	// # of selected clips in the Manager.
	int     fullsize;	// Space used.
	int     widgeth;	// Height of title bar + infoline.
	char    clipbrd[LBUFFER]; // Clipboard path.
	int     size;		// Size of clipbrd array.
	short   menuid;
	short	height;		// Height of Manager in Standard view mode.
	int     system;		// MINT or MAGIC.
	int     datesep;    // Date separator character.
	int     dformat;    // Date format.
	int     aeslang;
	TEXT   *text;		// Some textual data (mostly the GPL).
	char   avserver[9];
	GRECT   mgrrect;
	GRECT   cfgrect;	// Coordinates of config dialog.
	GRECT   infrect;	// Coordinates of Info dialog.
};

struct clips {
	char name[13];		// 8.3 GEMDOS style filename.
	char preview[36]; 	// Preview in the list box. Needed for switching between text and info mode.
	int  index;			// Index in the array of clips.
	int  select;		// Selected or not (TRUE/FALSE).
	long size;			// Size of file.
	_DATETIME time;		// Date and time info.
};

struct userconf { // The order mostly follows the items in the config dialog. See 'config.c' for details.
	short winx;
	short winy;
	int   iconify;
	int   autoicon;
	int   icontimeout;
	int   cleaning;
	int   viewmode;
	int   clipmode;
	int   order;    // Order in the list box.
	int   infomode;
	int   settings; // Remember window settings. TRUE or FALSE.
	int   cycling;
	int   bubbles;
	int   warnings;
	int   autoprotect;
	int   dbclick;
	int   rmbclick;
	int   enter;
	int   space;
	int   insert;
	int   size;
	long  clipsize;
	long  freespace;
	char  clipbrd[LBUFFER];
	int   override;
};

struct bubbles {
    char  *buffer;	// The text of all the bubble text strings.
    char **p;		// Pointers to each text strings.
};
